//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : camera
// created (by) : Christian Behrenberg, 2007-10-10
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// MAIN CAMERA ACTION
// Camera with an offset to the player (Rudi)
//--------------------------------------------------------------------
void cam_update ()
{
	#ifndef COMPILE
		// Stop here, if debug camera is on
			if (def_camera > 0) {return;}
	#endif
	
	// Position
		vec_set(vecTemp.x, cam_offset_target);
		vec_add(vecTemp.x, Rudi->x);

	// Interpolation
		vec_lerp(camera.x, camera.x, vecTemp.x, cam_interpolate_pos * time_step);

	// Angle

		// Calc a relative look-at point in front of the player
			vec_set(vecTemp, cam_std_lookAt);
			vec_rotate(vecTemp, Rudi->pan);
			vec_add(vecTemp, Rudi->x);

		// Look at there
			vec_diff(vecTemp.x, vecTemp.x, camera.x);
			vec_to_angle(vecTemp.x, vecTemp.x);

		// Interpolation
			ang_lerp(camera.pan, camera.pan, vecTemp.x, cam_interpolate_ang * time_step);
			camera.roll = 0;

	// Camera angle
		camera.arc += (cam_arc_target - camera.arc) * 0.1 * time_step;

	// Revert to default values
		cam_setOffset(cam_std_offset);
		cam_setArc(cam_std_arc);
}

//--------------------------------------------------------------------
// SETS AN ARC FOR THE CAMERA
//--------------------------------------------------------------------
void cam_setArc (var arc)
{
	cam_arc_target = arc;
}

//--------------------------------------------------------------------
// SETS AN OFFSET FOR THE CAMERA
//--------------------------------------------------------------------
void cam_setOffset (VECTOR* offset)
{
	vec_set(cam_offset_target, offset);
}