//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : credits
// created (by) : Christian Behrenberg, 2007-11-24
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// STARTS THE CREDITS
//--------------------------------------------------------------------
void cr_load ()
{
	// Set menu mode, hide all menu & HUD elements
		menu_mode = MENU_MODE_CREDITS;
		menu_hide_all();
		hud_hide();
		
	// Stop all music
		music_level_stop();
		music_menu_stop();
		
	// Unfreeze all
		freeze_mode = 0;
		
	// Load level (starts the credits)
		level_load(CR_LEVELFILE);
}

//--------------------------------------------------------------------
// CAMERA
//
// Moves the camera down along all elements. Begins, after the logo
// element is ready. Then moves down until a certain position deter-
// mined by the show-stopper element. Stops there.
//--------------------------------------------------------------------
void cr_camStart ()
{
	// Initialization

		cr_logoReady = 0;							// Logo flag
		set(my, INVISIBLE);						// Dummyentity should be invisible
		camera.arc = CR_CAM_ARC;				// Reset camera arc

		vec_set(sky_color, CR_SKY_COLOR);	// Solid sky

		vec_set(camera.x, my.x);				// Place & orientate camera
		vec_set(camera.pan, my.pan);

	// Synchronize
		wait(1);

	// Stop all (accidently) continued sounds
		snd_stopall(4);

	// Play credits music
		music_credits_start();

	// Wait until logo is ready
		while (!cr_logoReady) {wait(1);}

	// Move down as long we didn't reached the show-stopper heigth
		while (camera.z > cr_stop_z) {
			camera.z -= CR_CAM_ROLLSPEED * time_step;
			wait(1);
		}
}

//--------------------------------------------------------------------
// LOGO ELEMENT
//
// Is the first visible element on the screen. Fades in, stays a bit
// and then the credits-rolling will start.
//--------------------------------------------------------------------
void cr_logo ()
{
	// Initialization
		
		set(my, TRANSLUCENT);		// Alpha enabled
		my.alpha = 0;					// Invisible
		my.material = mat_cr_elem;	// Brighten up (=entity!)

	// Synchronize
		wait(1);

	// Center vertically
		my.z = camera.z;
		
	// On the XZ plane
		my.y = 0;

	// Wait a bit before I fade in
		wait(-CR_LOGO_WAITSTART);

	// Fade in
		while (my.alpha < 100) {
			my.alpha += CR_LOGO_FADESPEED * time_step;
			wait(1);
		}

	// Stay a bit
		wait(-CR_LOGO_STAY);

	// OK, lets start rolling down by signalizing readyness
		cr_logoReady = 1;

	// Fading behaviour like common elements
		cr_element();
		my.alpha = 100; //overwrite alpha
}

//--------------------------------------------------------------------
// COMMON ELEMENT
//
// Is invisible until the logo has been shown. Then, it fades in in
// relation to the position to the camera. If it is near enough, it
// fades in, otherwise it fades out. The fading is done by linear
// interpolation.
//--------------------------------------------------------------------
void cr_element ()
{
	// Initialization

		set(my, TRANSLUCENT);			// Alpha enabled
		my.alpha = 0;						// Invisible
		my.material = mat_cr_elem;		// Brighten up (=entity!)
		
	// On the XZ plane
		my.y = 0;

	// Synchronize
		wait(1);

	// Wait until logo has been shown
		while (!cr_logoReady) {wait(1);}

	// Fade according to the cam position
		while (1) {

			// Relative alpha depending on position to cam
				my.skill1 = (1 - (minv(abs(my.z - camera.z), CR_ELEM_FADEDIST) / CR_ELEM_FADEDIST)) * 100;

			// Absolute difference alpha
				my.skill2 = my.skill1 - my.alpha;

			// Interpolate from current to new alpha by 25%
				my.alpha += CR_ELEM_FADEFACTOR * time_step * my.skill2;

			wait(1);
		}
}

//--------------------------------------------------------------------
// SHOW-STOPPER
//
// Sets a height which will be evaluated by the camera. If the camera
// is below this height, it will stop rolling down.
//--------------------------------------------------------------------
void cr_stop ()
{
	cr_stop_z = my.z;		// Set show-stopper height
	ent_remove(my);		// Remove dummy entity
}

//--------------------------------------------------------------------
// "THANKS FOR PLAYING" - ELEMENT
//
// If the camera has been stopped (-> cr_stop), the "thanks for play-
// ing!" element will be shown. Afterwards, we go back to the menu.
//--------------------------------------------------------------------
void cr_tfp ()
{
	// Initialization

		set(my, TRANSLUCENT);		// Alpha-enabled
		my.alpha = 0;					// Invisible
		
		cr_tfp_ent = my;           // Register entity
		my.material = mat_cr_tfp;	// Brighten up (=entity!)
		
	// On the XZ plane
		my.y = 0;

	// Synchronize
		wait(1);

	// Wait until camera has stopped
		while (camera.z > cr_stop_z) {

			// If stopping is enabled and ESC is hit, break.
			// Stopping is only enabled, when credits were
			// accessed from the menu.

			if ((cr_stoppable) && (key_esc)) {

				// Go back to main menu

					level_load("");		// Unload level
					music_menu_start();	// Start menu music
					menu_main();			// Start menu
			}

			wait(1);
		}

	// Center the image vertically
		my.z = camera.z;

	// Fade in
		while (my.alpha < 100) {
			my.alpha += CR_TFP_FADEINSPEED * time_step;
			wait(1);
		}

	// Stay a bit
		wait(-CR_TFP_STAY);
	
	// Fade out
		while (my.alpha > 0) {
			my.alpha -= CR_TFP_FADEOUTSPEED * time_step;
			wait(1);
		}

	// Wait a bit
		wait(-CR_TFP_WAITAFTER);

	// Go back to main menu
		level_load("");		// Unload level
		music_menu_start();	// Start menu music
		menu_main();			// Start menu
}

//--------------------------------------------------------------------
// SNOWFLAKES
// As a gimmick, snowflakes will fly around
//--------------------------------------------------------------------
void cr_flake ()
{
	// Initialization

		// Enable facing
			vec_set(my.pan, nullvector);

		// skill1 is the rotation direction
		// skill2 is the rotation offset
	
			my.skill1 = 1 - random(2);	// ]-1 ... +1[
			my.skill2 = random(360);	// [0, 360[
	
		// Shiny!
			set(my, BRIGHT | TRANSLUCENT);
			
	// On the XZ plane, but behind elements
		my.y = -10;

	while (1) {

		// Movement is made of several additive sinus waves!
			my.z -= 0.1 + sin(total_ticks + my.skill2) * 0.1 * time_step;
			my.x += sin(total_ticks + my.skill2) * 0.5 * time_step;

		// Linear rotation
			my.roll += my.skill1 * 2 * time_step;

		// Inverse alpha relation if TFP sign. Useful, when the flake
		// is visible on the screen whily the TFP sign is visible at the
		// same time. While the TFP sign is fading in, the flake will
		// fade out!

			if (cr_tfp_ent) {
				my.alpha = 100 - cr_tfp_ent->alpha;
			}

		// When the flake is nearly invisible, remove it. Useful, because
		// when the TFP sign is fading out, the flake will fade in again,
		// which is not intended.

			if (my.alpha < 5) {
				break;
			}

		wait(1);
	}

	ent_remove(my);
}