//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// shader       : ocean shader
// created (by) : Christian Behrenberg, 2007-11-16
//--------------------------------------------------------------------

float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matWorldView;

float3 vecViewPos;
float4 vecTime;
float4 vecFog;

float	RippleScale = 0.8;		// Normalmap tiling
float RippleSpeed = 0.004;		// Waves speed

Texture entSkin1;	// Normalmap
Texture mtlSkin2; // Cubemap


sampler s_Bump = sampler_state
{
Texture = (entSkin1);
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU  = WRAP;
    AddressV  = WRAP;
};

sampler s_Cube = sampler_state
{
	Texture = (mtlSkin2);
	MinFilter = LINEAR;
   MagFilter = LINEAR;
   MipFilter = LINEAR;
   AddressU  = CLAMP;
   AddressV  = CLAMP;
   AddressW  = CLAMP;
};

struct VS_IN
{
   float4 Pos:  	POSITION;
   float3 Normal:	NORMAL;
   float2 Bump:		TEXCOORD0;
   float3 Tangent :	TEXCOORD2;
};


struct VS_OUT
{
   float4 Pos:  POSITION;
   float  Fog:  FOG;
   float2 Bump:	TEXCOORD0;
   float4 TanToCube0:	TEXCOORD1;
   float4 TanToCube1:	TEXCOORD2;
   float4 TanToCube2:	TEXCOORD3;
};


VS_OUT WaterBump_VS(VS_IN IN)
{
	VS_OUT Out;

	Out.Pos	= mul(IN.Pos, matWorldViewProj);
        
	//////////////////////////////////////////////////////////////////////////////// 
	// Ripple Test #1
	//
	Out.Bump= (IN.Bump.xy * RippleScale) + IN.Bump.xy + (vecTime.w * 0.1*RippleSpeed);
	////////////////////////////////////////////////////////////////////////////////
	

	//////////////////////////////////////////////////////////////////////////////// 
	// Ripple Test #2
	//
	//float cos_x = cos(RippleScale * vecTime.w);
   //    float sin_y = sin(RippleScale * vecTime.w);
	//Out.Bump = float2(IN.Bump.x+cos_x, IN.Bump.y-sin_y);
	////////////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////////// 
	// Ripple Test #3
	//
	//float cos_x=cos(vecTime.w*0.1*RippleSpeed);
	//float sin_y = sin(vecTime.w*0.1*RippleSpeed);
   //   Out.Bump = (float2(IN.Bump.x+cos_x, IN.Bump.y+sin_y)* RippleScale)*0.2;
	

	float3x3 ObjToTan;
	
	ObjToTan[0] = RippleScale * IN.Tangent;
	ObjToTan[1] = RippleScale * cross(IN.Tangent, IN.Normal);
	ObjToTan[2] = IN.Normal;

	Out.TanToCube0.xyz = mul(ObjToTan, matWorld[0].xyz);
	Out.TanToCube1.xyz = mul(ObjToTan, matWorld[1].xyz);
	Out.TanToCube2.xyz = mul(ObjToTan, matWorld[2].xyz);


	float3 PositionWorld = mul(IN.Pos, matWorld);
	float3 ViewerDir = normalize(vecViewPos - PositionWorld);

	Out.TanToCube0.w = ViewerDir.x;
	Out.TanToCube1.w = ViewerDir.y;
	Out.TanToCube2.w = ViewerDir.z;

	float ofog = 1 - (distance(PositionWorld, vecViewPos) - vecFog.x) * vecFog.z;
	Out.Fog = ofog;
		
	return Out;
}


technique WaterBump
{

	pass One
	{
		Sampler[0]	= (s_Bump);
		//Sampler[1]	= (s_Cube);
		//Sampler[2]	= (s_Cube);
		Sampler[3]	= (s_Cube);
		
		AlphaBlendEnable = True;
		ZwriteEnable = True;
		ZEnable = True;

		// x,y,z = RGB
		// w = alpha
		PixelShaderConstant[0] = {1, 1, 1, 0.60}; // x, y, z, w

		VertexShader = compile vs_1_1 WaterBump_VS();
		
		PixelShader =
		asm
		{
			ps.1.1

			tex t0		// Normalmap
					

			texm3x3pad   t1,  t0_bx2
			texm3x3pad   t2,  t0_bx2
			texm3x3vspec t3,  t0_bx2

			mul r0, t3, c0
			
		};
	}
}