//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// shader       : waving grass
// created (by) : Christian Behrenberg, 2007-11-25
//--------------------------------------------------------------------

texture   entSkin1;
float4x4  matWorldViewProj;
float4    vecTime;

#define speed 0.15	//Wind speed
#define range 20		//Wind force

sampler grassSampler = sampler_state 
{
	Texture = <entSkin1>;
	
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};



struct VS_INPUT
{
    float3 Pos      : POSITION; 
    float2 Tex      : TEXCOORD0;
	float2 Rotation : TEXCOORD2;
};

struct VS_OUTPUT
{
	float4 Pos      : POSITION;
	float2 Tex      : TEXCOORD0;
};



// --- Technique GrassPS
VS_OUTPUT vs_grass(VS_INPUT IN) 
{
	
	VS_OUTPUT OUT;
	float4 pos = float4(IN.Pos,1);


	if( IN.Tex.y <= 0.5 )
	{
		pos.x += sin(IN.Rotation.x * vecTime.w * speed)*range;
		pos.z += sin(IN.Rotation.y * vecTime.w * speed)*range;
	}

	OUT.Pos = mul(pos, matWorldViewProj);
	OUT.Tex = IN.Tex;
	
	return OUT;
}


float4 ps_grass(VS_OUTPUT IN) : COLOR 
{	
	return tex2D(grassSampler,IN.Tex);
}


// Techniques
technique grass 
{
	pass p0 
	{
		zWriteEnable=true;
		alphaTestEnable=true;
		VertexShader = compile vs_1_1 vs_grass();
		PixelShader = compile ps_1_1 ps_grass();
	}
}