//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : objects
// created (by) : Christian Behrenberg, 2007-11-01
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// LEVELGATE
// Opens, when no packets are left to collect.
//--------------------------------------------------------------------
void obj_gate ()
{
	// Initialization
		if (detail == DETAIL_HIGH) {
			fx_shiny(fx_cube_shiny);
		}

		set(my, POLYGON);				//enable collision
		reset(my, DYNAMIC);			//not dynamic

	// Create reset data
		reset_backupMe(my, obj_gate);
		
	obj_gateCam();
	
	wait(3);

	// Wait until last packet has been collected
		while (pk_count > obj_gate_threshold) {
			
			if (detail >= DETAIL_MEDIUM) {
				//	Sparkle effect
					obj_gate_sparkle();
			}

			if (reset_global) {
				reset_restoreMe(my);
				return;
			}

			wait(1);
		}

	// Effect
		eff_spawnEnt_verts(my, 64, eff_sparkMagic);

	// Open gate (animation)
		set(my, PASSABLE);
		var i;
		for (i = 0; i < 100; i += obj_gate_openAnim_speed * time_step) {
			ent_animate(my, obj_gate_openAnim_str, i, 0);

			if (reset_global) {
				reset_restoreMe(my);
				return;
			}

			wait(1);
		}
		
		snd_obj_gateBellPlay();

	// Effect
		eff_spawnEnt_verts(my, 64, eff_sparkBright);

	// It won't change anymore, so we disable dynamic
		reset(my, DYNAMIC);

	while (1) {

		if (detail >= DETAIL_MEDIUM) {
			//	Sparkle effect
				obj_gate_sparkle();
		}

		//bugfix
			ent_animate(my, obj_gate_openAnim_str, 100, 0);

		if (reset_global) {
			reset_restoreMe(my);
			return;
		}

		wait(1);
	}

}

	//- effect: sparkle -----------------------------------------------
	void obj_gate_sparkle ()
	{
		if (!(my.eflags & CLIPPED)) {

			if (my.skill1 <= 0) {
				eff_spawnEnt_verts(my, 2, eff_sparkBright);
				my.skill1 += 16;
			} else {
				my.skill1 -= time_step;
			}

		}
	}

	//- cam preset when Rudi is near enough ---------------------------
	void obj_gateCam ()
	{
		while (!Rudi) {wait(1);}

		while (my) {

			if (vec_dist(my.x, Rudi->x) < 750) {
				cam_setOffset(cam_gate_offset);
				cam_setArc(cam_gate_arc);
			}

			wait(1);
		}
	}

	//- environment model ---------------------------------------------
	void obj_gateEnvr ()
	{
		set(my, POLYGON);
		reset(my, DYNAMIC);
		fx_shiny(fx_cube_shiny);
	}
	
	//- levelgate trigger ---------------------------------------------
	void obj_gateTrig ()
	{
		set(my, PASSABLE | INVISIBLE); //no collision

		while (!Rudi) {wait(1);} //pointer validation

		while (vec_dist(my.x, Rudi->x) > my.max_x) {wait(1);} //wait until Rudi comes along

		// Deactivate Rudi. Death = 2 means: level finish
			Rudi->playerDeath = 2;
	}

//--------------------------------------------------------------------
// ESKIMO STUFF
//--------------------------------------------------------------------

	void obj_iglu ()
	{
		set(my, POLYGON);
		reset(my, DYNAMIC);
	}
	
	void obj_leverage ()
	{
		set(my, POLYGON);
		reset(my, DYNAMIC);
	}
	
	void obj_fishDead ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}
	
	
	void obj_fireOff ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}
	
	
	void obj_fireOn ()
	{
		set(my, PASSABLE);
		
		wait(2);
	
		snd_emitByEntDist(my, pl_listener, 2000, snd_obj_firePlay, snd_obj_fireStop);
	
		while (1) {
	
			if (!(my.eflags & CLIPPED)) {
	
				if (my.skill1 > 3) {
	
					if (detail >= DETAIL_MEDIUM) {
						
						effect(eff_bigFire_fire, 1, my.x, nullvector);
						effect(eff_bigFire_smoke, 1, my.x, nullvector);
					}
	
					my.skill1 -= 3;
	
				}
		
				my.skill1 += time_step;
				
			}
	
			wait(1);
		}
	}
	
//--------------------------------------------------------------------
// PENGUIN STUFF
//--------------------------------------------------------------------

	void obj_nest ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}
	
//--------------------------------------------------------------------
// ICEHOTEL
//--------------------------------------------------------------------

	void obj_hotelRoom ()
	{
		fx_shiny(fx_cube_shiny);
	
		set(my, POLYGON);
		reset(my, DYNAMIC);
	}
	
	void obj_hotelFront ()
	{
		set(my, POLYGON);
		reset(my, DYNAMIC);
	}
	
	void obj_hotelSign ()
	{
		set(my, POLYGON);
		reset(my, DYNAMIC);
	}
	
//--------------------------------------------------------------------
// SANTA STUFF
//--------------------------------------------------------------------

	void obj_santaSign ()
	{
		set(my, POLYGON);
		reset(my, DYNAMIC);
	}

	void obj_candyCane ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}
	
		void obj_candyCaneColl ()
		{
			set(my, POLYGON);
			reset(my, DYNAMIC);
		}
	
		void obj_candycaneColl ()
		{
			set(my, PASSABLE);
			reset(my, DYNAMIC);
		}
		
//--------------------------------------------------------------------
// ENVIRONMENT STUFF
//--------------------------------------------------------------------

	void obj_fence ()
	{
		set(my, POLYGON);
		reset(my, DYNAMIC);
	}
	
		void obj_fencePass ()
		{
			set(my, PASSABLE);
			reset(my, DYNAMIC);
		}
	
	void obj_snowman ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}
	
		void obj_snowmanColl ()
		{
			set(my, POLYGON);
			reset(my, DYNAMIC);
		}
		
	void obj_sign ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}		
		
//--------------------------------------------------------------------
// WATER STUFF
//--------------------------------------------------------------------

	void obj_iceA7 ()
	{
		set(my, PASSABLE);
		reset(my, DYNAMIC);
	}