//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : sledges
// created (by) : Christian Behrenberg, 2007-10-15
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// MAIN SLEDGE ACTION
//--------------------------------------------------------------------
void sl_main ()
{
	sl_init(); // Intialization

	while (my.sledgeDeath == 0) {

   	// Initialize the tracking calculation and do
		// the processing if no error occurs

	   	if (sl_tracking_init()) {
				sl_tracking_search();	//search position on track
				sl_move();					//move sledge
				sl_log();						//logging
			}
			
		// Environment check
			sl_check();

		//effects
			sl_sparkle();
			sl_eff_snowSplat();

			if (detail >= DETAIL_MEDIUM) {
				sl_rope();
			}

		wait(1);
	}

	sl_death();
}

	//--------------------------------------------------------------------
	// SLEDGE INITIALIZATION
	//--------------------------------------------------------------------
	void sl_init ()
	{
		set(my, PASSABLE);	//make passable (faster!)

		//log position
			vec_set(my.logPos, my.x);

		if (detail >= DETAIL_MEDIUM) {
			//effect: spark
				eff_spawnEnt_verts(my, 6, eff_sparkMagic);
		}

		wait(1);

		if (detail >= DETAIL_MEDIUM) {

			//rope

				if ((ENTITY*)(my.sledgeFront) == Rudi) {
					my.sledgeRopeEnt = ent_create("ropeToBridle.mdl", vector(0,0,-1000), sl_rope_ent);
				} else {
					my.sledgeRopeEnt = ent_create("ropeToSledge.mdl", vector(0,0,-1000), sl_rope_ent);
				}
		}
	}

	//--------------------------------------------------------------------
	// INITIALIZES THE PARAMETERS FOR THE TRACKING
	//
	// In the case if we are linked to Rudi or another sledge, we have to
	// use different search and positioning parameters, like the distance
	// to our parent, the parent position, the startpoint of the tracking
	// chain and the running distance we start with.
	//--------------------------------------------------------------------
	int sl_tracking_init ()
	{
		//Get the parent and if it exists, start intialization.

		you = (ENTITY*)(my.sledgeFront);
		if (you) {

				if (you == Rudi) { //We are the first sledge

					vec_set(my.sledgeParentPos, vector(Rudi->x, Rudi->y, Rudi->z + Rudi->min_z));
					my.sledgeDist = sl_dist_toRudi;
					my.sledgePoint = (void*)(trackRudi->first);
					my.sledgeTrackLength = 0;

				} else {

					//We are somewhere in the chain.
					//Our parent is another sledge

					vec_set(my.sledgeParentPos, you.x);
					my.sledgeDist = sl_dist_toSledge;
					my.sledgePoint = you.sledgeCtrlPoint;

					//parent sledge is somewhere in front of us an not AT the point
					my.sledgeTrackLength = vec_dist(((TRACKPOINT*)my.sledgePoint)->pos, you.x);
				}

			my.sledgeTrackLengthPrev = my.sledgeTrackLength;
			
			return(1); //everything is fine
		} else {
			return(0); //error
		}
	}

	//--------------------------------------------------------------------
	// SEARCHES THE CORRECT POSITION ON THE TRACK
	//--------------------------------------------------------------------
	void sl_tracking_search ()
	{
		// As long as we are too close to our parent,
		// move through the tracking chain

		while (my.sledgeTrackLength < my.sledgeDist) {
	
			//Log the distance from the previous iteration
			my.sledgeTrackLengthPrev = my.sledgeTrackLength;

			//We moved one step away, so we add the moved distance to the total moved distance
			my.sledgeTrackLength += vec_dist(((TRACKPOINT*)my.sledgePoint)->pos, my.sledgeParentPos);

			//Log current point so that my child can start tracking right here.
			//If NO child is available, we reached the end of the chain (exit)

			if (((TRACKPOINT*)my.sledgePoint)->child != 0) {
				my.sledgePoint = (void*)(((TRACKPOINT*)my.sledgePoint)->child);
			} else {
				break;
			}
		}
		
		//The current point is too far away and the parent point to narrow. We calculate
		//the right point between them now:

		//We only do this if we aren't the first point and if the difference of the
		//went track isn't to small (prevents division through zero later)
	
		if ((((TRACKPOINT*)my.sledgePoint)->parent != 0) &&				//not the first point
			((my.sledgeTrackLength - my.sledgeTrackLengthPrev) > 5))		//prevents division error
		{
			//We interpolate between both points. The result is the right position which fits the
			//predestined minimum distance. The factor is calculated by the ratio between the tracked
			//distance from the parent and current point and the searched distance.

			vec_lerp(my.sledgeTargetPos,	((TRACKPOINT*)my.sledgePoint)->parent->pos,
													((TRACKPOINT*)my.sledgePoint)->pos,
													((my.sledgeDist - my.sledgeTrackLengthPrev) /
													(my.sledgeTrackLength - my.sledgeTrackLengthPrev)));

		} else {

			//in the case that we are the first point, we directly set the position
			vec_set(my.sledgeTargetPos, ((TRACKPOINT*)my.sledgePoint)->pos);
		}
	}

	//--------------------------------------------------------------------
	// MOVES THE SLEDGE ACCORDING TO THE TRACK POSITION
	//--------------------------------------------------------------------
	void sl_move ()
	{
		// Move to the constraint position
			vec_diff(vecTemp.x, my.sledgeTargetPos, my.sledgeParentPos);
			vec_normalize(vecTemp.x, my.sledgeDist);
			vec_add(vecTemp.x, my.sledgeParentPos);

			vec_lerp(my.x, my.x, vecTemp.x, 0.9 * time_step);

		// Look to my parent
			vec_diff(vecTemp.x, my.sledgeParentPos, my.x);
			vec_to_angle(vecTemp.x, vecTemp.x);
			ang_lerp(my.pan, my.pan, vecTemp.x, 0.5 * time_step);
			my.roll = 0;
	}

	//--------------------------------------------------------------------
	// LOGS SOME USEFUL DATA
	//--------------------------------------------------------------------
	void sl_log ()
	{
		// Log control point
			my.sledgeCtrlPoint = (void*)(((TRACKPOINT*)my.sledgePoint)->child);

			vec_set(my.logPos, my.x);
	}

	//--------------------------------------------------------------------
	// DESTROYABLE CHECK
	//--------------------------------------------------------------------
	void sl_check ()
	{
		sl_check_cut();
		sl_check_frontDeath();
	}

		//- checks if Rudi cuts his chain ---------------------------------
		void sl_check_cut ()
		{
			if (Rudi) {
				//only if we are destroyable
				if (my.sledgeDestroyable) {
					if (vec_dist(Rudi->x, my.x) < sl_cutDist) {	//triggerrange
						sl_cut(my);
					}
				}
			}
		}

		//- checks if the front sledge is dead ----------------------------
		void sl_check_frontDeath ()
		{
			if (my.sledgeDestroyable) {

				// Check if the front sledge isn't dead

				you = (ENTITY*)my.sledgeFront;
				if (you != 0) {
					if (you.sledgeDeath != 0) {

						//front sledge is dead -> so I die, too
						my.sledgeDeath = 1;

					}
				} else {

					// I have no front sledge, so we die instantly
						my.sledgeDeath = 1;
				}
			}
		}

	//--------------------------------------------------------------------
	// SPARKLE EFFECT
	//--------------------------------------------------------------------
	void sl_sparkle ()
	{
		if (!(my.eflags & CLIPPED)) {
			if (my.skill1 <= 0) {
				eff_spawnEnt_verts(my, 1, eff_sparkBright);
				my.skill1 += 16 + random(32);
			} else {
				my.skill1 -= time_step;
			}
		}
	}

	//--------------------------------------------------------------------
	// ROPE ADJUSTMENT
	//--------------------------------------------------------------------
	void sl_rope ()
	{
		if (detail < DETAIL_MEDIUM) {return;}
		
		proc_mode = PROC_LATE;

		if (my.eflags & CLIPPED) {
			you = (ENTITY*)(my.sledgeRopeEnt);
			if (you) {
				set(you, INVISIBLE);
			}
			return;
		}

		// Get predecessor
		you = (ENTITY*)(my.sledgeFront);
		if (you) {
			
         	// Calculate my coupling point
				vec_set(vecTemp, vector(91.5, 0, 43));
				vec_rotate(vecTemp, my.pan);
				vec_add(vecTemp, my.x);

			// Set coupling point data for our predecessor
			if (you == Rudi) { 	// Rudi
				vec_set(vecTemp2, vector(-35, 0, 70));	//offset
				varTemp[0] = 78;								//rope length
			} else {					// Sledge
				vec_set(vecTemp2, vector(-25, 0, 30.3));
				varTemp[0] = 248;
			}

            // Calculate precedessor's coupling point
				vec_set(vecTemp2, vecTemp2);
				vec_rotate(vecTemp2, you.pan);
				vec_add(vecTemp2, you.x);

			// If predecessor is Rudi, adapt coupling to bridle
				if (you == Rudi) {
					vec_for_vertex(vecTemp3, (ENTITY*)(Rudi->playerBridle), 84);
					vecTemp2.z = vecTemp3.z;
				}

			// Get rope
			you = (ENTITY*)(my.sledgeRopeEnt);
			if (you) {

				// Sets the rope to my coupling point. Then we scale it, so
				// that the rope is long enough to couple my to my pred.
				// Then we rot. the rope to the pred. so that we're coupled
				
				reset(you, INVISIBLE);

         	vec_set(you.x, vecTemp);
				vec_diff(vecTemp3, vecTemp2, vecTemp);
				you.scale_x = vec_length(vecTemp3) / varTemp[0];
				vec_to_angle(you.pan, vecTemp3);

			}
		}
	}

		//- rope entity dummy function ------------------------------------
		void sl_rope_ent ()
		{
			set(my, PASSABLE);
		}

	//-----------------------------------------------------------------
	// DEATH
	//-----------------------------------------------------------------
	void sl_death ()
	{
		switch (my.sledgeDeath) {
			case 1:	sl_death_crash();	return;	//crash
			case 2:	sl_death_heart();	return;	//levelend -> hearts
		}
	}

		//- crash ------------------------------------------------------
		void sl_death_crash ()
		{
			if (!(my.eflags & CLIPPED)) {
				sl_eff_exploPacket();
			}
			
			if (detail >= DETAIL_MEDIUM) {
				ent_remove((ENTITY*)(my.sledgeRopeEnt));
			}
			
			ent_remove(my);
		}

		//- to hearts --------------------------------------------------
		void sl_death_heart ()
		{
			if (my.sledgeBack != 0) {
				sl_getHearts((ENTITY*)(my.sledgeBack));
			}
			
			if (vec_dist(my.x, Rudi->x) < 800) {

				//effects
					eff_spawnEnt_verts(my, 10, eff_sparkBright);
	
					vec_set(vecTemp.x, vector(0, 0, 145));
					vec_rotate(vecTemp.x, my.pan);
					vec_add(vecTemp.x, my.x);

					eff_sledgeHeart3D(vecTemp.x);
			}

			//add points
				points_toAdd += POINTS_SLEDGE;

			if (detail >= DETAIL_MEDIUM) {
				ent_remove((ENTITY*)(my.sledgeRopeEnt));
			}

			ent_remove(my);
		}

//--------------------------------------------------------------------
// CREATES INSTANTLY A NEW SLEDGE AND ADDS IT TO THE SLEDGE CHAIN
//--------------------------------------------------------------------
void sl_add ()
{
	// if there is no Rudi, there won't be sledges
	if (Rudi == NULL) {return;}

	// if a last sledge exists, add to sledge chain
	if (Rudi->sledgeLast) {

   	char* buffer[32];
   	sprintf(buffer, "sledge%c.mdl", 'A'+(int)(random(3)));
		entTemp[0] = ent_create(buffer, ((ENTITY*)Rudi->sledgeLast)->x, sl_main);

		entTemp[0]->sledgeFront = Rudi->sledgeLast;
		entTemp[1] = Rudi->sledgeLast;
		entTemp[1]->sledgeBack = entTemp[0];
		entTemp[0]->sledgeBack = 0;
		entTemp[0]->sledgeDestroyable = 1;
		Rudi->sledgeLast = entTemp[0];

		//orientate like parent
		vec_set(entTemp[0]->pan, ((ENTITY*)entTemp[0]->sledgeFront)->pan);

	} else {

		// Rudi has no sledges, so this will be the first one

     	char* buffer[32];
   	sprintf(buffer, "sledge%c.mdl", 'A'+(int)(random(3)));
		entTemp[0] = ent_create(buffer, Rudi->x, sl_main);

		Rudi->sledgeFirst = entTemp[0];
		Rudi->sledgeLast = Rudi->sledgeFirst;
		entTemp[0]->sledgeFront = Rudi;
		entTemp[0]->sledgeBack = 0;

		//orientate like Rudi
		vec_set(entTemp[0]->pan, Rudi->pan);
	}
}

//--------------------------------------------------------------------
// SPAWNS SLEDGES BUT AFTER ANOTHER AT THE END OF THE SLEDGE CHAIN
//--------------------------------------------------------------------
void sl_addNr (int nr)
{
	// if there is no Rudi, there won't be sledges
	if (Rudi == NULL) {return;}

 	int i;

	//spawn sledges after another..
	for (i = 0; i < nr; i++) {
		sl_add(); //spawn sledge
		wait(-0.2);
	}
}

//--------------------------------------------------------------------
// CUTS A CHAIN AT THE SLEDGE OF THIS POINTER
//--------------------------------------------------------------------
void sl_cut (ENTITY* sledge)
{
	if (sledge) {
		
		snd_pl_cut();

		Rudi->sledgeLast = sledge->sledgeFront;
		sledge->sledgeDeath = 1;
	}
}

//--------------------------------------------------------------------
// STARTS A CHAIN REACTION FOR SLEDGES TO HEARTS CONVERSION
//--------------------------------------------------------------------
void sl_getHearts (ENTITY* sledge)
{
	//If starting sledge is available
	if (sledge) {
		sledge->sledgeDeath = 2; //initialize chain reaction
	}
}

//--------------------------------------------------------------------
// EFFECTS
//--------------------------------------------------------------------

	//-----------------------------------------------------------------
	// exploding packets
	//-----------------------------------------------------------------
	void sl_eff_exploPacket ()
	{
		//choose from one of 3 sprite animations
			char entFile [128];
			sprintf(entFile, "effPacketExplo%c+8.tga", 'A' + (int)(random(3)));

		//create it: raise it by my height (because origin-z == ground)
			ent_create(entFile, vector(my.x, my.y, my.z + my.max_z), sl_eff_exploPacket_func);
			
		//some smoke puffs
			effect(eff_puff, 2, vector(my.x, my.y, my.z + my.max_z * 0.5), nullvector);
	}

		//- sprite function---------------------------------------------
		void sl_eff_exploPacket_func ()
		{
			//init
				set(my, PASSABLE);	//so that Rudi doesnt crash against me
				my.roll = 1;			//enable facing

			//different explosion scales
				vec_scale(my.scale_x, 2 + random(1.5));

			//one animation cycle; my.skill1 = animation counter
				while (my.skill1 < 100) {
					ent_animate(my, "", my.skill1, 0);
					my.skill1 += 30 * time_step;
					wait(1);
				}

			//animation is over, remove sprite
				ent_remove(my);
		}

	//-----------------------------------------------------------------
	// Snow splatter
	//-----------------------------------------------------------------
	void sl_eff_snowSplat ()
	{
		if (detail < DETAIL_HIGH) {return;}
		
		if (!(my.eflags & CLIPPED)) {

			my.skill2 -= time_step;
			if (my.skill2 < 0) {
	
				my.skill2 += 4;
	
				vec_set(vecTemp, vector(-random(75), -58, 3));
				vec_rotate(vecTemp, my.pan);
				vec_add(vecTemp, my.x);
	
				vec_set(vecTemp2, vector(-random(75), 58, 3));
				vec_rotate(vecTemp2, my.pan);
				vec_add(vecTemp2, my.x);
	
				effect(eff_snowSplat, 1, vecTemp, nullvector);
				effect(eff_snowSplat, 1, vecTemp2, nullvector);
			}
		}
	}