//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : system utilities
// created (by) : Christian Behrenberg, 2007-10-11
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// RESET & BACKUP FUNCTIONS
//--------------------------------------------------------------------

	//-----------------------------------------------------------------
	// CREATES BACKUP DATA FOR THE MY ENTITY
	//
	// params: myEnt       - my entity
	//         resetAction - function pointer of the entity action
	//-----------------------------------------------------------------
	void reset_backupMe (ENTITY* myEnt, void* resetAction)
	{
		if (myEnt) {
			vec_set(myEnt->resetData_pos, myEnt->x);
			vec_set(myEnt->resetData_rot, myEnt->pan);
			myEnt->resetData_ev = (void*)(resetAction);
		}
	}

	//-----------------------------------------------------------------
	// RESTORES A MY ENTITY
	// Requires a previous call of "reset_backupMe"!
	//
	// params: myEnt - my entity
	//-----------------------------------------------------------------
	void reset_restoreMe (ENTITY* myEnt)
	{
		if (myEnt) {
	
			you = ent_create(myEnt->type, myEnt->resetData_pos, (void*)(myEnt->resetData_ev));
		 	vec_set(you.pan, myEnt->resetData_rot);
			ent_remove(myEnt);
		}
	}

//--------------------------------------------------------------------
// ANGLE INTERPOLATION
// Like vec_lerp but for angles. Circumvents the gimbal lock issue.
//
// params:  resultVec   - vector which will receive the result angle
//          srcVec      - start angle of interpolation
//          destVec     - destionation angle of interpolation
//          blendFactor - [0..1], interpolation factor
//
// returns: result angle (as it will be copied into reultVec)
//--------------------------------------------------------------------
VECTOR* ang_lerp (VECTOR* resultVec, VECTOR* srcVec, VECTOR* destVec, var blendFactor)
{
	vec_set(resultVec, srcVec);						// Copy source into result
	vec_diff(ANG_LERP_VEC, destVec, resultVec);  // Difference vector

	ANG_LERP_VEC.x = ang(ANG_LERP_VEC.x);			// Adjust angle validity
	ANG_LERP_VEC.y = ang(ANG_LERP_VEC.y);
	ANG_LERP_VEC.z = ang(ANG_LERP_VEC.z);

	vec_scale(ANG_LERP_VEC.x, blendFactor);		// Interpolate the rotation
	vec_add(resultVec, ANG_LERP_VEC);				// Apply interpolated angle

	return(resultVec);									// Return result angle
}

//--------------------------------------------------------------------
// BITMAP CHECK
// Checks a bmap ptr and if it is invalid, the bitmap will be created
//
// params:  source - bitmap ptr
//          file   - file which will be loaded if source is invalid
//
// returns: already existing bitmap ptr (=source) or if source is not
//          valid, the bmap ptr of the newly created file.
//--------------------------------------------------------------------
BMAP* bmapCheck(BMAP* source, char* file)
{
	// If source bitmap is invalid
		if (source == 0) {

			// Create bmap and write it into source
				source = bmap_create(file);
		}

	return(source); // Return created/existing bmap
}

//--------------------------------------------------------------------
// CREATES A PANEL
// Instead of pan_create, pan_createEx uses advanced parameters.
//
// params:  content - char* which holds the layout elements
//          layer   - panel layer
//          bitmap  - char* to a file which shall be loaded if != NULL
//          posX    - initial hor. position
//          posY    - initial ver. position
//          flags   - flags which shall be set when panel is created
//
// returns: panel pointer
//--------------------------------------------------------------------
PANEL* pan_createEx (char* content, var layer, char* bitmap, int posX, int posY, long flags)
{
	// Create standard panel
		PANEL* tmpPnl = pan_create(content, layer);

	// If a string has been passed, use "bitmap" to
	// load the bitmap file
		if (bitmap != NULL) {
			tmpPnl->bmap = bmap_create(bitmap);				// Create bitmap
			tmpPnl->size_x = bmap_width(tmpPnl->bmap);	// Adjust size
			tmpPnl->size_y = bmap_height(tmpPnl->bmap);
		}

	// Pose panel
		tmpPnl->pos_x = posX;
		tmpPnl->pos_y = posY;

	// Set flags
		set(tmpPnl, flags);

	return(tmpPnl); // Return panel pointer
}