//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : track recording
// created (by) : Christian Behrenberg, 2007-10-15
//--------------------------------------------------------------------

 //--------------------------------------------------------------------
// CREATES & RETURNS A NEW TRACK
//
// params: initPos  - Vector with initial position data. Most often
//                    the position of the recorder entity is passed.
//         nrPoints - Number of Trackpoints to be created. If <= 0,
//                    only the empty object will be returned.
//--------------------------------------------------------------------
TRACK* create_TRACK (VECTOR* initPos, int nrPoints)
{
	TRACK* temp = malloc(sizeof(TRACK));

	temp->last = 0;
	temp->first = 0;

	if (nrPoints > 0) {
		temp->first = create_TRACKPOINT(initPos);
		temp->last = temp->first;

		int i;
		for (i = 1; i < nrPoints; i++) {
			add_TRACK_point(temp);
		}
	}

	return(temp);
}

//--------------------------------------------------------------------
// CREATES & RETURNS A TRACKPOINT
//
// params: initPos - initial position of the track point
//--------------------------------------------------------------------
TRACKPOINT* create_TRACKPOINT (VECTOR* initPos)
{
	//allocation
		TRACKPOINT* temp = malloc(sizeof(TRACKPOINT));
		temp->pos = malloc(sizeof(VECTOR));

	//initialization
		vec_set(temp->pos, initPos);
		temp->parent = 0;
		temp->child = 0;

	return(temp);
}

//--------------------------------------------------------------------
// REMOVES A TRACK OBJECT
//
// params: track - track chain
//--------------------------------------------------------------------
void remove_TRACK (TRACK* track)
{
	if (track == 0) {return;}

	TRACKPOINT* point = track->first;
	TRACKPOINT* point_next;

	while (point != 0) {
		free(point->pos);
		point_next = point->child;
		free(point);
		point = point_next;
	}

	free(track);
	track = 0;
}

//--------------------------------------------------------------------
// ADDS A TRACKPOINT
// The trackpoint will be added in a track-chain after the last point.
//
// params: track - track chain
//--------------------------------------------------------------------
void add_TRACK_point (TRACK* track)
{
	track->last->child = create_TRACKPOINT(track->last->pos);
	track->last->child->parent = track->last;
	track->last = track->last->child;
}

//--------------------------------------------------------------------
// REGISTERS A NEW POINT & CONVOLUTES CHAIN
//
// params: track - track chain
//         pos   - new point to be registered
//--------------------------------------------------------------------
void reg_TRACK_point (TRACK* track, VECTOR* pos)
{
	// We need at least 2 points, which is indicated by
	// different last/first pointers:

	if (track->last != track->first) {

		// Set new position
			vec_set(track->last->pos, pos);

		// Push last point to the front of the chain and modify the pointers
		// so that the chain is repaired

			track->last->child = track->first;
			track->last->parent->child = 0;
			track->first->parent = track->last;
			track->first = track->last;
			track->last = track->last->parent;
			track->last->child = 0;
	}
}

//--------------------------------------------------------------------
// DBEUG FUNCTION
// Will infinitely draw the chain with red dots for each track point.
// Call it only once!
//
// params: track - track chain to be displayed
//--------------------------------------------------------------------
void debug_TRACK (TRACK* track)
{
	TRACKPOINT* point = track->first;

	// Go through the chain and draw
	// each point with a big red dot

		while (point != 0) {
			draw_point3d(point->pos, vector(0,0,255), 0, 8);
			point = point->child;
		}
}