//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : track recording -- HEADER
// created (by) : Christian Behrenberg, 2007-10-15
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// TRACK RECORDING DATA STRUCTURE
//--------------------------------------------------------------------
//	FIGURE:      last       first
//				     |           |
//	points: 0 <- (A) - (B) - (C) -> 0
//             child <-|-> parent
//
// time:   "older" ...   ... "newer"
//--------------------------------------------------------------------

	// STORES A POSITION IN 3D SPACE & IS PART OF A CHAIN OF POINTS
	//
	// The parent point is a newer record point, whereas the child
	// point is older than the current

		typedef struct TRACKPOINT {
			VECTOR* pos;
			struct TRACKPOINT *parent, *child;
		} TRACKPOINT;

	// STORES A LINKED LIST OF TRACKPOINTS TO STORE A CHANGING PATH
	//
	// Stores a path which is recorded by a moving entity. The "first"
	// trackpoint is at the recording entity, whereas the "last" entity
	// is the oldest recorded trackpoint. If a new trackpoint is
	// recorded, the position is written into the last trackpoint and
	// this point will be moved in front of the first point to become
	// the new "first" point, the second last point will be the new
	// "last" point

		typedef struct TRACK {
			TRACKPOINT *last;
			TRACKPOINT *first;
		} TRACK;

//--------------------------------------------------------------------

//--------------------------------------------------------------------
// PROTOTYPES
//--------------------------------------------------------------------

	TRACK*		create_TRACK (VECTOR* initPos, int nrPoints);
	TRACKPOINT*	create_TRACKPOINT (VECTOR* initPos);
	void			remove_TRACK (TRACK* track);
	void			add_TRACK_point (TRACK* track);
	void			reg_TRACK_point (TRACK* track, VECTOR* pos);
	void			debug_TRACK (TRACK* track);